import { Action, AnyAction, Reducer, Store } from 'redux'
export declare const persistTypes: {
  /** Rehydrate an reducer */
  readonly REHYDRATE: 'persist/rehydrate'
  /** All reducers were completely rehydrated */
  readonly COMPLETE: 'persist/complete'
  /** Purge an reducer state */
  readonly PURGE: 'persist/purge'
  /** Pause an reducer persistence */
  readonly PAUSE: 'persist/pause'
  /** Resume an reducer persistence */
  readonly RESUME: 'persist/resume'
}
declare type PersistStorage = {
  setItem: (key: string, value: any) => Promise<any>
  getItem: (key: string) => Promise<any>
}
export declare type PersistConfig = {
  /** Persist key (name in storage, persist key in action) */
  key: string
  /** A storage object to save state */
  storage?: PersistStorage
  /** A function to serialize state to storage */
  serialize?: (state: any) => string
  /** A function to deserialize state from storage */
  deserialize?: (stored: string) => any
  /** A function to compare states */
  compare?: <S>(state: S, oldState: S, action: AnyAction) => boolean
  /** A function to merge states */
  merge?: <S>(stored: S, state: S) => S
  /** Throttle save in ms */
  throttle?: number
}
export declare function persistReducer<S = any, A extends Action = AnyAction>(
  config: PersistConfig,
  reducer: Reducer<S, A>,
): (state: any, action: any) => any
/** @param storage a default storage */
export declare function persistStore(
  store: Store,
  storage?: PersistStorage,
): {
  rehydrate: () => Promise<{
    type: 'persist/complete'
  }>
  then: any
  purge: (key?: string) => void
  pause: (key?: string) => void
  resume: (key?: string) => void
}
export declare const localStorage: {
  setItem: (key: any, value: any) => Promise<unknown>
  getItem: (key: any) => Promise<unknown>
  removeItem: (key: any) => Promise<unknown>
  clear: () => Promise<unknown>
}
export declare const sessionStorage: {
  setItem: (key: any, value: any) => Promise<unknown>
  getItem: (key: any) => Promise<unknown>
  removeItem: (key: any) => Promise<unknown>
  clear: () => Promise<unknown>
}
export declare const memoryStorage: {
  setItem: (key: any, value: any) => Promise<void>
  getItem: (key: any) => Promise<any>
}
export {}
